@@ -30,10 +30,11 @@ Since CycloneDX schema version 1.4, this has become part of the core schema.
 
 
 import re
+from collections.abc import Iterable
 from datetime import datetime
 from decimal import Decimal
 from enum import Enum
-from typing import Any, Dict, FrozenSet, Iterable, Optional, Tuple, Type, Union
+from typing import Any, Optional, Union
 
 import py_serializable as serializable
 from sortedcontainers import SortedSet
@@ -167,7 +168,7 @@ class BomTarget:
         versions: Optional[Iterable[BomTargetVersionRange]] = None,
     ) -> None:
         self.ref = ref
-        self.versions = versions or []  # type:ignore[assignment]
+        self.versions = versions or []
 
     @property
     @serializable.xml_sequence(1)
@@ -235,11 +236,15 @@ class VulnerabilityAnalysis:
         justification: Optional[ImpactAnalysisJustification] = None,
         responses: Optional[Iterable[ImpactAnalysisResponse]] = None,
         detail: Optional[str] = None,
+        first_issued: Optional[datetime] = None,
+        last_updated: Optional[datetime] = None,
     ) -> None:
         self.state = state
         self.justification = justification
-        self.responses = responses or []  # type:ignore[assignment]
+        self.responses = responses or []
         self.detail = detail
+        self.first_issued = first_issued
+        self.last_updated = last_updated
 
     @property
     @serializable.xml_sequence(1)
@@ -307,29 +312,36 @@ class VulnerabilityAnalysis:
     def detail(self, detail: Optional[str]) -> None:
         self._detail = detail
 
-    # @property
-    # @serializable.view(SchemaVersion1Dot5)
-    # @serializable.xml_sequence(5)
-    # def first_issued(self) -> ...:
-    #     ...  # TODO since CDX 1.5
-    #
-    # @first_issued.setter
-    # def first_issued(self, ...) -> None:
-    #     ...  # TODO since CDX 1.5
+    @property
+    @serializable.view(SchemaVersion1Dot5)
+    @serializable.view(SchemaVersion1Dot6)
+    @serializable.type_mapping(serializable.helpers.XsdDateTime)
+    @serializable.xml_sequence(5)
+    def first_issued(self) -> Optional[datetime]:
+        return self._first_issued
 
-    # @property
-    # @serializable.view(SchemaVersion1Dot5)
-    # @serializable.xml_sequence(6)
-    # def last_updated(self) -> ...:
-    #     ...  # TODO since CDX 1.5
-    #
-    # @last_updated.setter
-    # def last_updated(self, ...) -> None:
-    #     ...  # TODO since CDX 1.5
+    @first_issued.setter
+    def first_issued(self, first_issue: Optional[datetime]) -> None:
+        self._first_issued = first_issue
+
+    @property
+    @serializable.view(SchemaVersion1Dot5)
+    @serializable.view(SchemaVersion1Dot6)
+    @serializable.type_mapping(serializable.helpers.XsdDateTime)
+    @serializable.xml_sequence(6)
+    def last_updated(self) -> Optional[datetime]:
+        return self._last_updated
+
+    @last_updated.setter
+    def last_updated(self, last_updated: Optional[datetime]) -> None:
+        self._last_updated = last_updated
 
     def __comparable_tuple(self) -> _ComparableTuple:
         return _ComparableTuple((
-            self.state, self.justification, tuple(self.responses), self.detail
+            self.state, self.justification,
+            _ComparableTuple(self.responses),
+            self.detail,
+            self.first_issued, self.last_updated
         ))
 
     def __eq__(self, other: object) -> bool:
@@ -337,6 +349,11 @@ class VulnerabilityAnalysis:
             return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
+    def __lt__(self, other: Any) -> bool:
+        if isinstance(other, VulnerabilityAnalysis):
+            return self.__comparable_tuple() < other.__comparable_tuple()
+        return NotImplemented
+
     def __hash__(self) -> int:
         return hash(self.__comparable_tuple())
 
@@ -594,14 +611,17 @@ class VulnerabilityScoreSource(str, Enum):
             Always returns an instance of `VulnerabilityScoreSource`. `VulnerabilityScoreSource.OTHER` is
             returned if the scheme is not obvious or known to us.
         """
+        if vector.startswith('CVSS:4.'):
+            return VulnerabilityScoreSource.CVSS_V4
         if vector.startswith('CVSS:3.'):
+            if vector.startswith('CVSS:3.1'):
+                return VulnerabilityScoreSource.CVSS_V3_1
             return VulnerabilityScoreSource.CVSS_V3
-        elif vector.startswith('CVSS:2.'):
+        if vector.startswith('CVSS:2.'):
             return VulnerabilityScoreSource.CVSS_V2
-        elif vector.startswith('OWASP'):
+        if vector.startswith('OWASP'):
             return VulnerabilityScoreSource.OWASP
-        else:
-            return VulnerabilityScoreSource.OTHER
+        return VulnerabilityScoreSource.OTHER
 
     def get_localised_vector(self, vector: str) -> str:
         """
@@ -613,15 +633,16 @@ class VulnerabilityScoreSource(str, Enum):
         Returns:
             The vector without any scheme prefix as a `str`.
         """
-        if self == VulnerabilityScoreSource.CVSS_V3 and vector.startswith('CVSS:3.'):
-            return re.sub('^CVSS:3\\.\\d/?', '', vector)
-
-        if self == VulnerabilityScoreSource.CVSS_V2 and vector.startswith('CVSS:2.'):
-            return re.sub('^CVSS:2\\.\\d/?', '', vector)
-
-        if self == VulnerabilityScoreSource.OWASP and vector.startswith('OWASP'):
-            return re.sub('^OWASP/?', '', vector)
-
+        if self is VulnerabilityScoreSource.CVSS_V4 and vector.startswith('CVSS:4.'):
+            return re.sub(r'^CVSS:4\.\d/?', '', vector)
+        if (
+            self in (VulnerabilityScoreSource.CVSS_V3_1, VulnerabilityScoreSource.CVSS_V3)
+        ) and vector.startswith('CVSS:3.'):
+            return re.sub(r'^CVSS:3\.\d/?', '', vector)
+        if self is VulnerabilityScoreSource.CVSS_V2 and vector.startswith('CVSS:2.'):
+            return re.sub(r'^CVSS:2\.\d/?', '', vector)
+        if self is VulnerabilityScoreSource.OWASP and vector.startswith('OWASP'):
+            return re.sub(r'^OWASP/?', '', vector)
         return vector
 
     def get_value_pre_1_4(self) -> str:
@@ -632,7 +653,7 @@ class VulnerabilityScoreSource(str, Enum):
         Returns:
             `str`
         """
-        if self == VulnerabilityScoreSource.OWASP:
+        if self is VulnerabilityScoreSource.OWASP:
             return 'OWASP Risk'
         return self.value  # type:ignore[no-any-return]
 
@@ -640,7 +661,7 @@ class VulnerabilityScoreSource(str, Enum):
 class _VulnerabilityScoreSourceSerializationHelper(serializable.helpers.BaseHelper):
     """  THIS CLASS IS NON-PUBLIC API  """
 
-    __CASES: Dict[Type[serializable.ViewType], FrozenSet[VulnerabilityScoreSource]] = dict()
+    __CASES: dict[type[serializable.ViewType], frozenset[VulnerabilityScoreSource]] = dict()
     __CASES[SchemaVersion1Dot4] = frozenset({
         VulnerabilityScoreSource.CVSS_V2,
         VulnerabilityScoreSource.CVSS_V3,
@@ -655,7 +676,7 @@ class _VulnerabilityScoreSourceSerializationHelper(serializable.helpers.BaseHelp
     __CASES[SchemaVersion1Dot6] = __CASES[SchemaVersion1Dot5]
 
     @classmethod
-    def __normalize(cls, vss: VulnerabilityScoreSource, view: Type[serializable.ViewType]) -> str:
+    def __normalize(cls, vss: VulnerabilityScoreSource, view: type[serializable.ViewType]) -> str:
         return (
             vss
             if vss in cls.__CASES.get(view, ())
@@ -664,14 +685,14 @@ class _VulnerabilityScoreSourceSerializationHelper(serializable.helpers.BaseHelp
 
     @classmethod
     def json_normalize(cls, o: Any, *,
-                       view: Optional[Type[serializable.ViewType]],
+                       view: Optional[type[serializable.ViewType]],
                        **__: Any) -> str:
         assert view is not None
         return cls.__normalize(o, view)
 
     @classmethod
     def xml_normalize(cls, o: Any, *,
-                      view: Optional[Type[serializable.ViewType]],
+                      view: Optional[type[serializable.ViewType]],
                       **__: Any) -> str:
         assert view is not None
         return cls.__normalize(o, view)
@@ -698,7 +719,7 @@ class VulnerabilitySeverity(str, Enum):
     UNKNOWN = 'unknown'
 
     @staticmethod
-    def get_from_cvss_scores(scores: Union[Tuple[float, ...], float, None]) -> 'VulnerabilitySeverity':
+    def get_from_cvss_scores(scores: Union[tuple[float, ...], float, None]) -> 'VulnerabilitySeverity':
         """
         Derives the Severity of a Vulnerability from it's declared CVSS scores.
 
@@ -887,8 +908,8 @@ class VulnerabilityCredits:
         organizations: Optional[Iterable[OrganizationalEntity]] = None,
         individuals: Optional[Iterable[OrganizationalContact]] = None,
     ) -> None:
-        self.organizations = organizations or []  # type:ignore[assignment]
-        self.individuals = individuals or []  # type:ignore[assignment]
+        self.organizations = organizations or []
+        self.individuals = individuals or []
 
     @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'organization')
@@ -982,22 +1003,22 @@ class Vulnerability:
         self._bom_ref = _bom_ref_from_str(bom_ref)
         self.id = id
         self.source = source
-        self.references = references or []  # type:ignore[assignment]
-        self.ratings = ratings or []  # type:ignore[assignment]
-        self.cwes = cwes or []  # type:ignore[assignment]
+        self.references = references or []
+        self.ratings = ratings or []
+        self.cwes = cwes or []
         self.description = description
         self.detail = detail
         self.recommendation = recommendation
         self.workaround = workaround
-        self.advisories = advisories or []  # type:ignore[assignment]
+        self.advisories = advisories or []
         self.created = created
         self.published = published
         self.updated = updated
         self.credits = credits
-        self.tools = tools or []  # type:ignore[assignment]
+        self.tools = tools or []
         self.analysis = analysis
-        self.affects = affects or []  # type:ignore[assignment]
-        self.properties = properties or []  # type:ignore[assignment]
+        self.affects = affects or []
+        self.properties = properties or []
 
     @property
     @serializable.json_name('bom-ref')
